@@ -20,8 +20,12 @@ module Annex.FileMatcher (
 	preferredContentParser,
 	ParseToken,
 	parsedToMatcher,
-	mkLargeFilesParser,
+	mkMatchExpressionParser,
 	largeFilesMatcher,
+	AddUnlockedMatcher,
+	addUnlockedMatcher,
+	checkAddUnlockedMatcher,
+	module Types.FileMatcher
 ) where
 
 import qualified Data.Map as M
@@ -30,12 +34,14 @@ import Annex.Common
 import Limit
 import Utility.Matcher
 import Types.Group
-import qualified Annex
 import Types.FileMatcher
+import Types.GitConfig
+import Config.GitConfig
 import Git.FilePath
 import Types.Remote (RemoteConfig)
 import Annex.CheckAttr
 import Git.CheckAttr (unspecifiedAttr)
+import qualified Git.Config
 #ifdef WITH_MAGICMIME
 import Annex.Magic
 #endif
@@ -54,7 +60,7 @@ checkFileMatcher' getmatcher file notconfigured = do
 	matcher <- getmatcher file
 	checkMatcher matcher Nothing afile S.empty notconfigured d
   where
-	afile = AssociatedFile (Just file)
+	afile = AssociatedFile (Just (toRawFilePath file))
 	-- checkMatcher will never use this, because afile is provided.
 	d = return True
 
@@ -72,7 +78,7 @@ checkMatcher' :: FileMatcher Annex -> MatchInfo -> AssumeNotPresent -> Annex Boo
 checkMatcher' matcher mi notpresent =
 	matchMrun matcher $ \a -> a notpresent mi
 
-fileMatchInfo :: FilePath -> Annex MatchInfo
+fileMatchInfo :: RawFilePath -> Annex MatchInfo
 fileMatchInfo file = do
 	matchfile <- getTopFilePath <$> inRepo (toTopFilePath file)
 	return $ MatchingFile FileInfo
@@ -173,8 +179,8 @@ preferredContentTokens pcd = concat
 preferredContentParser :: [ParseToken (MatchFiles Annex)] -> String -> [ParseResult (MatchFiles Annex)]
 preferredContentParser tokens = map (parseToken tokens) . tokenizeMatcher
 
-mkLargeFilesParser :: Annex (String -> [ParseResult (MatchFiles Annex)])
-mkLargeFilesParser = do
+mkMatchExpressionParser :: Annex (String -> [ParseResult (MatchFiles Annex)])
+mkMatchExpressionParser = do
 #ifdef WITH_MAGICMIME
 	magicmime <- liftIO initMagicMime
 	let mimer n f = ValueToken n (usev $ f magicmime)
@@ -197,26 +203,57 @@ mkLargeFilesParser = do
 		]
 #endif
 	return $ map parse . tokenizeMatcher
-  where
 
 {- Generates a matcher for files large enough (or meeting other criteria)
- - to be added to the annex, rather than directly to git. -}
+ - to be added to the annex, rather than directly to git.
+ -
+ - annex.largefiles is configured in git config, or git attributes,
+ - or global git-annex config, in that order.
+ -}
 largeFilesMatcher :: Annex GetFileMatcher
-largeFilesMatcher = go =<< annexLargeFiles <$> Annex.getGitConfig
+largeFilesMatcher = go =<< getGitConfigVal' annexLargeFiles
   where
-	go (Just expr) = do
-		matcher <- mkmatcher expr
+	go (HasGitConfig (Just expr)) = do
+		matcher <- mkmatcher expr "git config"
 		return $ const $ return matcher
-	go Nothing = return $ \file -> do
+	go v = return $ \file -> do
 		expr <- checkAttr "annex.largefiles" file
 		if null expr || expr == unspecifiedAttr
-			then return matchAll
-			else mkmatcher expr
+			then case v of
+				HasGlobalConfig (Just expr') ->
+					mkmatcher expr' "git-annex config"
+				_ -> return matchAll
+			else mkmatcher expr "gitattributes"
 
-	mkmatcher expr = do
-		parser <- mkLargeFilesParser
-		either badexpr return $ parsedToMatcher $ parser expr
-	badexpr e = giveup $ "bad annex.largefiles configuration: " ++ e
+	mkmatcher expr cfgfrom = do
+		parser <- mkMatchExpressionParser
+		either (badexpr cfgfrom) return $ parsedToMatcher $ parser expr
+	badexpr cfgfrom e = giveup $ "bad annex.largefiles configuration in " ++ cfgfrom ++ ": " ++ e
+
+newtype AddUnlockedMatcher = AddUnlockedMatcher (FileMatcher Annex)
+
+addUnlockedMatcher :: Annex AddUnlockedMatcher
+addUnlockedMatcher = AddUnlockedMatcher <$> 
+	(go =<< getGitConfigVal' annexAddUnlocked)
+  where
+	go (HasGitConfig (Just expr)) = mkmatcher expr "git config"
+	go (HasGlobalConfig (Just expr)) = mkmatcher expr "git annex config"
+	go _ = matchalways False
+
+	mkmatcher :: String -> String -> Annex (FileMatcher Annex)
+	mkmatcher expr cfgfrom = case Git.Config.isTrueFalse expr of
+		Just b -> matchalways b
+		Nothing -> do
+			parser <- mkMatchExpressionParser
+			either (badexpr cfgfrom) return $ parsedToMatcher $ parser expr
+	badexpr cfgfrom e = giveup $ "bad annex.addunlocked configuration in " ++ cfgfrom ++ ": " ++ e
+
+	matchalways True = return $ MOp limitAnything
+	matchalways False = return $ MOp limitNothing
+
+checkAddUnlockedMatcher :: AddUnlockedMatcher -> MatchInfo -> Annex Bool
+checkAddUnlockedMatcher (AddUnlockedMatcher matcher) mi = 
+	checkMatcher' matcher mi S.empty
 
 simply :: MatchFiles Annex -> ParseResult (MatchFiles Annex)
 simply = Right . Operation
