@@ -2,7 +2,7 @@
  -
  - Copyright 2016 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 module Command.CalcKey where
@@ -10,6 +10,7 @@ module Command.CalcKey where
 import Command
 import Backend (genKey)
 import Types.KeySource
+import Utility.Metered
 
 cmd :: Command
 cmd = noCommit $ noMessages $ dontCheck repoExists $
@@ -19,7 +20,7 @@ cmd = noCommit $ noMessages $ dontCheck repoExists $
 		(batchable run (pure ()))
 
 run :: () -> String -> Annex Bool
-run _ file = genKey (KeySource file file Nothing) Nothing >>= \case
+run _ file = genKey (KeySource file file Nothing) nullMeterUpdate Nothing >>= \case
 	Just (k, _) -> do
 		liftIO $ putStrLn $ serializeKey k
 		return True
