@@ -2,7 +2,7 @@
  -
  - Copyright 2011-2016 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 module Command.Reinject where
@@ -12,6 +12,7 @@ import Logs.Location
 import Annex.Content
 import Backend
 import Types.KeySource
+import Utility.Metered
 
 cmd :: Command
 cmd = command "reinject" SectionUtility 
@@ -41,31 +42,30 @@ seek os
 startSrcDest :: [FilePath] -> CommandStart
 startSrcDest (src:dest:[])
 	| src == dest = stop
-	| otherwise = notAnnexed src $ do
-		showStart "reinject" dest
-		next $ ifAnnexed dest go stop
+	| otherwise = notAnnexed src $ ifAnnexed (toRawFilePath dest) go stop
   where
-	go key = ifM (verifyKeyContent RetrievalAllKeysSecure DefaultVerify UnVerified key src)
-		( perform src key
-		, giveup $ src ++ " does not have expected content of " ++ dest
-		)
+	go key = starting "reinject" (ActionItemOther (Just src)) $
+		ifM (verifyKeyContent RetrievalAllKeysSecure DefaultVerify UnVerified key src)
+			( perform src key
+			, giveup $ src ++ " does not have expected content of " ++ dest
+			)
 startSrcDest _ = giveup "specify a src file and a dest file"
 
 startKnown :: FilePath -> CommandStart
-startKnown src = notAnnexed src $ do
-	showStart "reinject" src
-	mkb <- genKey (KeySource src src Nothing) Nothing
-	case mkb of
-		Nothing -> error "Failed to generate key"
-		Just (key, _) -> ifM (isKnownKey key)
-			( next $ perform src key
-			, do
-				warning "Not known content; skipping"
-				next $ next $ return True
-			)
+startKnown src = notAnnexed src $
+	starting "reinject" (ActionItemOther (Just src)) $ do
+		mkb <- genKey (KeySource src src Nothing) nullMeterUpdate Nothing
+		case mkb of
+			Nothing -> error "Failed to generate key"
+			Just (key, _) -> ifM (isKnownKey key)
+				( perform src key
+				, do
+					warning "Not known content; skipping"
+					next $ return True
+				)
 
 notAnnexed :: FilePath -> CommandStart -> CommandStart
-notAnnexed src = ifAnnexed src $
+notAnnexed src = ifAnnexed (toRawFilePath src) $
 	giveup $ "cannot used annexed file as src: " ++ src
 
 perform :: FilePath -> Key -> CommandPerform
