@@ -2,7 +2,7 @@
  -
  - Copyright 2014 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 module Command.VPop where
@@ -16,7 +16,7 @@ import Logs.View
 import Command.View (checkoutViewBranch)
 
 cmd :: Command
-cmd = notBareRepo $ notDirect $
+cmd = notBareRepo $
 	command "vpop" SectionMetaData "switch back to previous view"
 		paramNumber (withParams seek)
 
@@ -27,17 +27,16 @@ start :: [String] -> CommandStart
 start ps = go =<< currentView
   where
 	go Nothing = giveup "Not in a view."
-	go (Just v) = do
-		showStart' "vpop" (Just $ show num)
+	go (Just v) = starting "vpop" (ActionItemOther (Just $ show num)) $ do
 		removeView v
 		(oldvs, vs) <- splitAt (num - 1) . filter (sameparentbranch v)
 			<$> recentViews
 		mapM_ removeView oldvs
 		case vs of
-			(oldv:_) -> next $ next $ do
+			(oldv:_) -> next $ do
 				showOutput
 				checkoutViewBranch oldv (return . branchView)
-			_ -> next $ next $ do
+			_ -> next $ do
 				showOutput
 				inRepo $ Git.Command.runBool
 					[ Param "checkout"
