@@ -2,13 +2,14 @@
  -
  - Copyright 2015 Joey Hess <id@joeyh.name>
  -:
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 {-# LANGUAGE QuasiQuotes, TypeFamilies, TemplateHaskell #-}
 {-# LANGUAGE OverloadedStrings, GADTs, FlexibleContexts #-}
 {-# LANGUAGE MultiParamTypeClasses, GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE UndecidableInstances #-}
 
 module Database.Fsck (
 	FsckHandle,
@@ -82,7 +83,7 @@ addDb (FsckHandle h _) k = H.queueDb h checkcommit $
 		| sz > 1000 = return True
 		| otherwise = do
 			now <- getCurrentTime
-			return $ diffUTCTime lastcommittime now > 300
+			return $ diffUTCTime now lastcommittime > 300
 
 {- Doesn't know about keys that were just added with addDb. -}
 inDb :: FsckHandle -> Key -> IO Bool
