@@ -17,6 +17,7 @@ import Data.Maybe
 import Data.Char
 import qualified Data.ByteString as S
 import qualified Data.Text as T
+import Control.DeepSeq
 
 import Utility.PartialPrelude
 import Key
@@ -41,6 +42,9 @@ derivePersistField "SKey"
 -- work when it's used in any kind of complex data structure.
 newtype IKey = IKey String
 
+instance NFData IKey where
+	rnf (IKey s) = rnf s
+
 instance Read IKey where
 	readsPrec _ s = [(IKey s, "")]
 
